@@ -1,7 +1,7 @@
 /*
  *  csync2 - cluster synchronization tool, 2nd generation
  *  LINBIT Information Technologies GmbH <http://www.linbit.com>
- *  Copyright (C) 2004, 2005  Clifford Wolf <clifford@clifford.at>
+ *  Copyright (C) 2004, 2005, 2006  Clifford Wolf <clifford@clifford.at>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -82,6 +82,7 @@ enum {
 	MODE_TEST_SYNC,
 	MODE_LIST_DIRTY,
 	MODE_REMOVE_OLD,
+	MODE_COMPARE,
 	MODE_SIMPLE
 };
 
@@ -101,7 +102,8 @@ PACKAGE_STRING " - cluster synchronization tool, 2nd generation\n"
 "	-h [-r] file..		Add (recursive) hints for check to db\n"
 "	-c [-r] file..		Check files and maybe add to dirty db\n"
 "	-u [-d] [-r] file..	Updates files if listed in dirty db\n"
-"	-f file..		Force this file in sync (resolve conflict)\n"
+"	-o [-r] file..		Create list of files in compare-mode\n"
+"	-f [-r] file..		Force this file in sync (resolve conflict)\n"
 "	-m file..		Mark files in database as dirty\n"
 "\n"
 "Simple mode:\n"
@@ -163,6 +165,7 @@ PACKAGE_STRING " - cluster synchronization tool, 2nd generation\n"
 "\n"
 "	-P peer1,peer1,...\n"
 "		Only update this peers (still mark all as dirty).\n"
+"		Only show files for this peers in -o (compare) mode.\n"
 "\n"
 "	-F	Add new entries to dirty database with force flag set.\n"
 "\n"
@@ -171,7 +174,7 @@ PACKAGE_STRING " - cluster synchronization tool, 2nd generation\n"
 "	-s filename\n"
 "		Print timestamps also to this file.\n"
 "\n"
-"	-W fd	Write a list of directories in which relevant file can be\n"
+"	-W fd	Write a list of directories in which relevant files can be\n"
 "		found to the specified file descriptor (when doing a -c run).\n"
 "		The directory names in this output are zero-terminated.\n"
 "\n"
@@ -290,7 +293,7 @@ int main(int argc, char ** argv)
 		return 1;
 	}
 
-	while ( (opt = getopt(argc, argv, "W:s:Ftp:G:P:C:D:N:HBAIXULSTMRvhcuimfxrd")) != -1 ) {
+	while ( (opt = getopt(argc, argv, "W:s:Ftp:G:P:C:D:N:HBAIXULSTMRvhcuoimfxrd")) != -1 ) {
 		switch (opt) {
 			case 'W':
 				csync_dump_dir_fd = atoi(optarg);
@@ -366,6 +369,10 @@ int main(int argc, char ** argv)
 					mode = MODE_UPDATE;
 				}
 				break;
+			case 'o':
+				if ( mode != MODE_NONE ) help(argv[0]);
+				mode = MODE_COMPARE;
+				break;
 			case 'i':
 				if ( mode == MODE_INETD )
 					mode = MODE_SERVER;
@@ -428,6 +435,7 @@ int main(int argc, char ** argv)
 			mode != MODE_HINT && mode != MODE_MARK &&
 			mode != MODE_FORCE && mode != MODE_SIMPLE &&
 			mode != MODE_UPDATE && mode != MODE_CHECK &&
+			mode != MODE_COMPARE &&
 			mode != MODE_CHECK_AND_UPDATE &&
 			mode != MODE_LIST_SYNC && mode != MODE_TEST_SYNC)
 		help(argv[0]);
@@ -450,9 +458,6 @@ int main(int argc, char ** argv)
 	for (i=0; myhostname[i]; i++)
 		myhostname[i] = tolower(myhostname[i]);
 
-	for (i=optind; i < argc; i++)
-		on_cygwin_lowercase(argv[i]);
-
 	/* Stand-alone server mode. This is a hack..
 	 */
 	if ( mode == MODE_SERVER || mode == MODE_SINGLE ) {
@@ -477,12 +482,17 @@ int main(int argc, char ** argv)
 		para = cmd ? strtok(0, "\t \r\n") : 0;
 
 		if (cmd && !strcasecmp(cmd, "ssl")) {
+#ifdef HAVE_LIBGNUTLS_OPENSSL
 			conn_printf("OK (activating_ssl).\n");
 			conn_activate_ssl(1);
 
 			if ( !conn_gets(line, 4096) ) return 0;
 			cmd = strtok(line, "\t \r\n");
 			para = cmd ? strtok(0, "\t \r\n") : 0;
+#else
+			conn_printf("This csync2 server is built without SSL support.\n");
+			return 0;
+#endif
 		}
 
 		if (!cmd || strcasecmp(cmd, "config")) {
@@ -523,8 +533,19 @@ int main(int argc, char ** argv)
 	yyparse();
 	fclose(yyin);
 
+	{
+		const struct csync_group *g;
+		for (g=csync_group; g; g=g->next)
+			if ( g->myname ) goto found_a_group;
+		csync_fatal("This host (%s) is not a member of any configured group.\n", myhostname);
+found_a_group:;
+	}
+
 	csync_db_open(file_database);
 
+	for (i=optind; i < argc; i++)
+		on_cygwin_lowercase(argv[i]);
+
 	switch (mode) {
 		case MODE_SIMPLE:
 			if ( argc == optind )
@@ -604,6 +625,15 @@ int main(int argc, char ** argv)
 			}
 			break;
 
+		case MODE_COMPARE:
+			csync_compare_mode = 1;
+			for (i=optind; i < argc; i++) {
+				char *realname = getrealfn(argv[i]);
+				csync_check_usefullness(realname, recursive);
+				csync_check(realname, recursive, init_run);
+			}
+			break;
+
 		case MODE_INETD:
 			conn_printf("OK (cmd_finished).\n");
 			csync_daemon_session();
@@ -694,7 +724,8 @@ int main(int argc, char ** argv)
 			} SQL_END;
 			break;
 
-		case MODE_TEST_SYNC:
+		case MODE_TEST_SYNC: {
+			char *realname;
 			if (init_run && init_run_with_removals)
 				init_run |= 2;
 			if (init_run && init_run_straight)
@@ -702,10 +733,14 @@ int main(int argc, char ** argv)
 			switch (argc-optind)
 			{
 			case 3:
-				if ( mode_test_auto_diff )
-					retval = csync_diff(argv[optind], argv[optind+1], argv[optind+2]);
-				else
-					if ( csync_insynctest(argv[optind], argv[optind+1], init_run, 0, argv[optind+2]) )
+				realname = getrealfn(argv[optind+2]);
+				csync_check_usefullness(realname, 0);
+
+				if ( mode_test_auto_diff ) {
+					csync_compare_mode = 1;
+					retval = csync_diff(argv[optind], argv[optind+1], realname);
+				} else
+					if ( csync_insynctest(argv[optind], argv[optind+1], init_run, 0, realname) )
 						retval = 2;
 				break;
 			case 2:
@@ -713,7 +748,12 @@ int main(int argc, char ** argv)
 					retval = 2;
 				break;
 			case 1:
-				if ( csync_insynctest_all(init_run, mode_test_auto_diff, argv[optind]) )
+				realname = getrealfn(argv[optind]);
+				csync_check_usefullness(realname, 0);
+
+				if ( mode_test_auto_diff )
+					csync_compare_mode = 1;
+				if ( csync_insynctest_all(init_run, mode_test_auto_diff, realname) )
 					retval = 2;
 				break;
 			case 0:
@@ -722,6 +762,7 @@ int main(int argc, char ** argv)
 				break;
 			}
 			break;
+		}
 
 		case MODE_LIST_DIRTY:
 			retval = 2;
